Stan Math Library: stan/math/prim/scal/prob/normal_cdf.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

normal_cdf.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_NORMAL_CDF_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_NORMAL_CDF_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/value_of.hpp>
13 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
14 #include <stan/math/prim/scal/meta/max_size.hpp>
15 #include <boost/random/normal_distribution.hpp>
16 #include <boost/random/variate_generator.hpp>
17 #include <cmath>
18
19
20 namespace stan {
21
22 namespace math {
23
38 template <typename T_y, typename T_loc, typename T_scale>
39 typename return_type<T_y, T_loc, T_scale>::type
40 normal_cdf(const T_y& y, const T_loc& mu, const T_scale& sigma) {
41 static const char* function("stan::math::normal_cdf");
42 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
43 T_partials_return;
44
45 using stan::math::check_positive;
46 using stan::math::check_finite;
47 using stan::math::check_not_nan;
48 using stan::math::value_of;
49 using stan::math::check_consistent_sizes;
50 using stan::math::INV_SQRT_2;
51 using std::exp;
52
53 T_partials_return cdf(1.0);
54
55 // check if any vectors are zero length
56 if (!(stan::length(y)
57 && stan::length(mu)
58 && stan::length(sigma)))
59 return cdf;
60
61 check_not_nan(function, "Random variable", y);
62 check_finite(function, "Location parameter", mu);
63 check_not_nan(function, "Scale parameter", sigma);
64 check_positive(function, "Scale parameter", sigma);
65 check_consistent_sizes(function,
66 "Random variable", y,
67 "Location parameter", mu,
68 "Scale parameter", sigma);
69
70
71 OperandsAndPartials<T_y, T_loc, T_scale>
72 operands_and_partials(y, mu, sigma);
73
74 VectorView<const T_y> y_vec(y);
75 VectorView<const T_loc> mu_vec(mu);
76 VectorView<const T_scale> sigma_vec(sigma);
77 size_t N = max_size(y, mu, sigma);
78 const double SQRT_TWO_OVER_PI = std::sqrt(2.0 / stan::math::pi());
79
80 for (size_t n = 0; n < N; n++) {
81 const T_partials_return y_dbl = value_of(y_vec[n]);
82 const T_partials_return mu_dbl = value_of(mu_vec[n]);
83 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
84 const T_partials_return scaled_diff = (y_dbl - mu_dbl)
85 / (sigma_dbl * SQRT_2);
86 T_partials_return cdf_;
87 if (scaled_diff < -37.5 * INV_SQRT_2)
88 cdf_ = 0.0;
89 else if (scaled_diff < -5.0 * INV_SQRT_2)
90 cdf_ = 0.5 * erfc(-scaled_diff);
91 else if (scaled_diff > 8.25 * INV_SQRT_2)
92 cdf_ = 1;
93 else
94 cdf_ = 0.5 * (1.0 + erf(scaled_diff));
95
96 // cdf
97 cdf *= cdf_;
98
99 // gradients
100 if (contains_nonconstant_struct<T_y, T_loc, T_scale>::value) {
101 const T_partials_return rep_deriv
102 = scaled_diff < -37.5 * INV_SQRT_2
103 ? 0.0
104 : SQRT_TWO_OVER_PI * 0.5
105 * exp(-scaled_diff * scaled_diff) / cdf_ / sigma_dbl;
106 if (!is_constant_struct<T_y>::value)
107 operands_and_partials.d_x1[n] += rep_deriv;
108 if (!is_constant_struct<T_loc>::value)
109 operands_and_partials.d_x2[n] -= rep_deriv;
110 if (!is_constant_struct<T_scale>::value)
111 operands_and_partials.d_x3[n] -= rep_deriv * scaled_diff * SQRT_2;
112 }
113 }
114
115 if (!is_constant_struct<T_y>::value) {
116 for (size_t n = 0; n < stan::length(y); ++n)
117 operands_and_partials.d_x1[n] *= cdf;
118 }
119 if (!is_constant_struct<T_loc>::value) {
120 for (size_t n = 0; n < stan::length(mu); ++n)
121 operands_and_partials.d_x2[n] *= cdf;
122 }
123 if (!is_constant_struct<T_scale>::value) {
124 for (size_t n = 0; n < stan::length(sigma); ++n)
125 operands_and_partials.d_x3[n] *= cdf;
126 }
127
128 return operands_and_partials.value(cdf);
129 }
130 }
131 }
132 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::sqrt
fvar< T > sqrt(const fvar< T > &x)
Definition: sqrt.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::erf
fvar< T > erf(const fvar< T > &x)
Definition: erf.hpp:14
is_constant_struct.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::SQRT_2
const double SQRT_2
The value of the square root of 2, .
Definition: constants.hpp:21
stan::math::INV_SQRT_2
const double INV_SQRT_2
The value of 1 over the square root of 2, .
Definition: constants.hpp:27
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::check_positive
bool check_positive(const char *function, const char *name, const T_y &y)
Return true if y is positive.
Definition: check_positive.hpp:68
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
stan::contains_nonconstant_struct
Definition: contains_nonconstant_struct.hpp:14
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::erfc
fvar< T > erfc(const fvar< T > &x)
Definition: erfc.hpp:14
stan::math::pi
double pi()
Return the value of pi.
Definition: constants.hpp:86
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
stan::math::normal_cdf
return_type< T_y, T_loc, T_scale >::type normal_cdf(const T_y &y, const T_loc &mu, const T_scale &sigma)
Calculates the normal cumulative distribution function for the given variate, location, and scale.
Definition: normal_cdf.hpp:40
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
max_size.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.